#ifndef UTILITIEC_REGEX_MACHINE_STATESTRUCT_H
#define UTILITIEC_REGEX_MACHINE_STATESTRUCT_H

#include "../StringView/StringView.h"

enum RegexMachineStateType {
	// Accept out of a Character class
	REGEXMACHINESTATETYPE_ACCEPTCLASS,
	// Accept a character (sequence)
	REGEXMACHINESTATETYPE_ACCEPT,
	// Capture group
	REGEXMACHINESTATETYPE_GROUP,
	// Accept one of the following
	REGEXMACHINESTATETYPE_OPTION,
	// Repeat the contained node
	REGEXMACHINESTATETYPE_REPEAT,
};

typedef struct RegexMachineStateBase_s {
	// Type of this node
	enum RegexMachineStateType type;

	// Next regex node, if this one matched
	struct RegexMachineStateBase_s* next;
} RegexMachineStateBase;

typedef struct RegexMachineStateOption_s {
	RegexMachineStateBase base;

	// next option
	struct RegexMachineStateOption_s* next_option;
} RegexMachineStateOption;

typedef struct RegexMachineStateAccept_s {
	RegexMachineStateBase base;

	// allocated in the scratchpad buffer
	StringView sequence;
} RegexMachineStateAccept;

typedef struct RegexMachineStateGroup_s {
	RegexMachineStateBase base;

	// zero=root group
	// SIZE_MAX=noncapturing group
	size_t number;

	// NULL=unnamed
	char* name;
} RegexMachineStateGroup;

typedef struct RegexMachineStateRepeat_s {
	RegexMachineStateBase base;

	size_t min;
	size_t max;
	
	bool greedy;
	bool possessive;

	RegexMachineStateBase* repeatable;
} RegexMachineStateRepeat;



#endif
