#include "os_rwlock.h"
#include <pthread.h>

#if defined(__unix__) || (defined(__APPLE__) && defined(__MACH__)) // Fuck you, apple

int OSRWLock_Create(OSRWLock* target)
{
	return pthread_rwlock_init(target, NULL);
}

void OSRWLock_Destroy(OSRWLock* rwlock)
{
	pthread_rwlock_destroy(rwlock);
}

int OSRWLock_Read(OSRWLock* rwlock)
{
	return pthread_rwlock_rdlock(rwlock);
}

int OSRWLock_Write(OSRWLock* rwlock)
{
	return pthread_rwlock_wrlock(rwlock);
}

int OSRWLock_ReleaseRead(OSRWLock* rwlock)
{
	return pthread_rwlock_unlock(rwlock);
}

int OSRWLock_ReleaseWrite(OSRWLock* rwlock)
{
	return pthread_rwlock_unlock(rwlock);
}

#elif defined(_WIN32)

int OSRWLock_Create(OSRWLock* target)
{
	InitializeSRWLock(*target);

	return EXIT_SUCCESS;
}

void OSRWLock_Destroy(OSRWLock* rwlock)
{
	// Win32 Locks can be forgotten if there are no more threads waiting on it
	OSRWLock_Write(*rwlock);
	// Now, nobody else can be holding the lock
	OSRWLock_ReleaseWrite(*rwlock);

	// Forget about it
	return;
}

int OSRWLock_Read(OSRWLock* rwlock)
{
	AcquireSRWLockShared(*rwlock);

	return EXIT_SUCCESS;
}

int OSRWLock_Write(OSRWLock* rwlock)
{
	AcquireSRWLockExclusive(*rwlock);

	return EXIT_SUCCESS;
}


int OSRWLock_ReleaseRead(OSRWLock* rwlock)
{
	ReleaseSRWLockShared(*rwlock);

	return EXIT_SUCCESS;
}

int OSRWLock_ReleaseWrite(OSRWLock* rwlock)
{
	ReleaseSRWLockExclusive(*rwlock);

	return EXIT_SUCCESS;
}


#else

#error "Threading standard not supported"

#endif
